clear all; close all; clc

%% Simulation of a dynamic process, the stirred-tank, using explicit 
%  Euler integration 
% 
%  In this exercise we are going to simulate the time-evolution of 
%  the stirred-tank, starting from some given initial conditions of 
%  the state variables


%
%  We simulate the process model when subjected to 3 different scenarios
%
%  Case 0. Initial state non-zero, A constant zero controls, 
%           -> Initially non-zero volume V(0)!=0 and temperature T(0)!=0
%           -> Fixed at zero flow rate w_i(t), temperature T_i(t) 
%              and heat flow rate Q(t)
%              w_i(t) = 0, T_i(t) = 0, Q(t) = 0
%
%  Case 1. Initial state zero, whatever constant controls,
%           -> Initially non-zero volume V(0)!=0 and temperature T(0)!=0
%           -> Fixed flow rate w_i(t), temperature T_i(t)
%              and heat flow rate Q(t)
%              w_i(t) = const, T_i(t) = const, Q(t) = const
%
%  Case 2. Time-varying control signal, initial state whatever
%           -> Whatever initial volume V(0) and temperature T(0)
%           -> Whatever dynamic flow-rate w_i(t), temperature T_i(t)
%              and heat flow rate Q(t)

%% Define the parameters of the system
paramater1 = 1
paramater2 = 1

    p = [paramater1; paramater2]; % Plug them into a 2x1 vector of parameters

%% Define the simulation time (t0 to tF, in deltaTime long intervals)
    tF = 10;                        % Final time of the simulation (min)
    deltaTime = 1/60;               % Step size (min)
    
    nSteps = tF/deltaTime;          % Number of time steps between T0 and Tf (adim)
    nSteps = round(nSteps);         % Round to closest integer 

%% Pick a case to simulate
caseN = input('Type in case number (0, 1, or 2): '); 

switch caseN    
    case 0
    %% Case 0.
 
    case 1
    %% Case 1.

    case 2
    %% Case 2. 
end

%% Simulation the time-evolution of the process, under the selected case

% Initialise the solution, to be replaced during simulation
    V_1 = zeros(1,nSteps);          % nSteps values of state variable 1
    V_1(1) = V_1_at_0;              % Assign initial value, known

    V_2 = zeros(1,nSteps);          % nSteps values of state variable 2
    V_2(1) = T_at_0;              % Assign initial values, known  

    X = [V_1;V_2];                  % Into a 2 x nSteps matrix (control notation)

% Initialise the measurements, to be replaces during simulation 
    Y = ;

% Simulate the time evolution of the two-tank system
%      fprintf('\n== Simulation ==\nitime | time (min) |  V_1  |  V_2  |  q_J  |  T_F  | dV_1/dt | dV_2/dt \n')
    for itime = 2:nSteps
        
        % Sanity checks to remove negative volumes in the tanks
        for ix = 1:numel(X0) 
            if X(ix,itime-1) < 0 
               X(ix,itime-1) = 0; 
            end
        end 
        
        % Sanity check to limit T_F x deltaTime to be at most what's inside  tank, V_1
        %if U(2,itime-1) >= X(1,itime-1)/deltaTime
        %   aU(2,itime-1) = X(1,itime-1)/deltaTime;
        %end    
        
        % Compute new value of the state variables (dynamics)

        X(:,itime) = X(:,itime-1) + deltaTime * stirredTank_template(1,X(:,itime-1),aU(:,itime-1),p);
        
%          fprintf('%3d   | %-10.2f | %-3.3f | %-3.3f | %-3.3f | %-3.3f | %-7.2f | %-7.2f \n', ...
%           itime-1, (itime-1)*deltaTime, X(1,itime-1), X(2,itime-1), U(1,itime-1), U(2,itime-1), ...
%           twoTanks(1,X(:,itime-1),aU(:,itime-1),p)')
%          pause(0.01)
        
        % Compute the new value of the output variable (read-out)
        Y(:,itime) = ;
    end
    
% Plot simulation results  
    plotStirredTank_template(U,aU,X,Y,p,nSteps,deltaTime)











